import 'dart:async';
import 'package:bytedesk_kefu/model/codeResult.dart';
import 'package:bytedesk_kefu/model/jsonResult.dart';
import 'package:bytedesk_kefu/model/oauth.dart';
import 'package:bloc/bloc.dart';
// import 'package:meta/meta.dart';
// import 'package:rxdart/rxdart.dart';
import 'package:bytedesk_kefu/blocs/login_bloc/bloc.dart';
import 'package:bytedesk_kefu/repositories/repositories.dart';

class LoginBloc extends Bloc<LoginEvent, LoginState> {
  //
  final UserRepository _userRepository = new UserRepository();

  LoginBloc() : super(LoginInitial());

  @override
  Stream<LoginState> mapEventToState(LoginEvent event) async* {
    //
    if (event is LoginButtonPressed) {
      //
      yield* _mapLoginState(
        username: event.username,
        password: event.password,
      );
    } else if (event is SMSLoginButtonPressed) {
      //
      yield* _mapSMSLoginState(
        mobile: event.mobile,
        code: event.code,
      );
    } else if (event is RegisterButtonPressed) {
      //
      yield* _mapRegisterState(
        mobile: event.mobile,
        password: event.password,
      );
    } else if (event is RequestCodeButtonPressed) {
      //
      yield* _mapRequestCodeState(
        mobile: event.mobile,
      );
    } else if (event is ResetPasswordButtonPressed) {
      //
      yield* _mapResetPasswordState(
        mobile: event.mobile,
        password: event.password,
      );
    } else if (event is UpdatePasswordButtonPressed) {
      //
      yield* _mapUpdatePasswordState(
        mobile: event.mobile,
        password: event.password,
      );
    }
  }

  Stream<LoginState> _mapLoginState({String username, String password}) async* {
    // print("LoginButtonPressed $username, $password ");
    //
    yield LoginInProgress();
    try {
      //
      OAuth oauth = await _userRepository.login(username, password);
      if (oauth.statusCode == 200) {
        // 用户名密码正确，登录成功
        yield LoginSuccess();
      } else {
        // 用户名密码错误，登录失败
        yield LoginError();
      }
    } catch (error) {
      // 网络或其他错误
      yield LoginFailure(error: error.toString());
    }
  }

  Stream<LoginState> _mapSMSLoginState({String mobile, String code}) async* {
    // print("LoginButtonPressed $username, $password ");
    //
    yield LoginInProgress();
    try {
      //
      OAuth oauth = await _userRepository.smsOAuth(mobile, code);
      if (oauth.statusCode == 200) {
        // 用户名密码正确，登录成功
        yield SMSLoginSuccess();
      } else {
        // 用户名密码错误，登录失败
        yield LoginError();
      }
    } catch (error) {
      // 网络或其他错误
      yield LoginFailure(error: error.toString());
    }
  }

  Stream<LoginState> _mapRegisterState(
      {String mobile, String password}) async* {
    //
    yield RegisterInProgress();
    try {
      //
      JsonResult jsonResult = await _userRepository.register(mobile, password);
      if (jsonResult.statusCode == 200) {
        yield RegisterSuccess();
      } else {
        yield RegisterError(
            message: jsonResult.message, statusCode: jsonResult.statusCode);
      }
    } catch (error) {
      // 网络或其他错误
      yield LoginFailure(error: error.toString());
    }
  }

  Stream<LoginState> _mapRequestCodeState({String mobile}) async* {
    // print("LoginButtonPressed $username, $password ");
    //
    yield RequestCodeInProgress();
    try {
      //
      CodeResult codeResult = await _userRepository.requestCode(mobile);
      if (codeResult.statusCode == 200) {
        yield RequestCodeSuccess();
      } else {
        yield RequestCodeError(
            message: codeResult.message, statusCode: codeResult.statusCode);
      }
    } catch (error) {
      // 网络或其他错误
      yield LoginFailure(error: error.toString());
    }
  }

  Stream<LoginState> _mapResetPasswordState(
      {String mobile, String password}) async* {
    // print("LoginButtonPressed $username, $password ");
    //
    yield ResetPasswordInProgress();
    try {
      //
      JsonResult jsonResult =
          await _userRepository.changePassword(mobile, password);
      if (jsonResult.statusCode == 200) {
        yield ResetPasswordSuccess();
      } else {
        yield ResetPasswordError(
            message: jsonResult.message, statusCode: jsonResult.statusCode);
      }
    } catch (error) {
      // 网络或其他错误
      yield LoginFailure(error: error.toString());
    }
  }

  Stream<LoginState> _mapUpdatePasswordState(
      {String mobile, String password}) async* {
    // print("LoginButtonPressed $username, $password ");
    //
    yield UpdatePasswordInProgress();
    try {
      //
      JsonResult jsonResult =
          await _userRepository.changePassword(mobile, password);
      if (jsonResult.statusCode == 200) {
        yield UpdatePasswordSuccess();
      } else {
        yield UpdatePasswordError(
            message: jsonResult.message, statusCode: jsonResult.statusCode);
      }
    } catch (error) {
      // 网络或其他错误
      yield LoginFailure(error: error.toString());
    }
  }
}
