import 'dart:async';
import 'package:bytedesk_kefu/model/requestThread.dart';
import 'package:bloc/bloc.dart';
// import 'package:flutter/foundation.dart';
import './bloc.dart';
import 'package:bytedesk_kefu/repositories/repositories.dart';
import 'package:bytedesk_kefu/model/model.dart';
// import 'package:http/http.dart' as http;

class ThreadBloc extends Bloc<ThreadEvent, ThreadState> {
  final ThreadRepository threadRepository = new ThreadRepository();

  ThreadBloc() : super(ThreadEmpty());

  @override
  Stream<ThreadState> mapEventToState(ThreadEvent event) async* {
    //
    if (event is InitThreadEvent) {
      yield InitialThreadState();
    } else if (event is RefreshThreadEvent) {
      yield* _mapRefreshThreadToState(event);
    } else if (event is RefreshVisitorThreadEvent) {
      yield* _mapRefreshVisitorThreadToState(event);
    } else if (event is RequestThreadEvent) {
      yield* _mapRequestThreadToState(event);
    } else if (event is RequestAgentEvent) {
      yield* _mapRequestAgentToState(event);
    }
  }

  Stream<ThreadState> _mapRefreshThreadToState(
      RefreshThreadEvent event) async* {
    yield ThreadLoading();
    try {
      final List<Thread> threadList = await threadRepository.getThreads();
      yield ThreadLoadSuccess(threadList);
    } catch (error) {
      print(error);
      yield ThreadLoadError();
    }
  }

  Stream<ThreadState> _mapRefreshVisitorThreadToState(
      RefreshVisitorThreadEvent event) async* {
    yield ThreadLoading();
    try {
      final List<Thread> threadList =
          await threadRepository.getVisitorThreads(event.page, event.size);
      yield ThreadLoadSuccess(threadList);
    } catch (error) {
      print(error);
      yield ThreadLoadError();
    }
  }

  Stream<ThreadState> _mapRequestThreadToState(
      RequestThreadEvent event) async* {
    yield ThreadLoading();
    try {
      final RequestThreadResult thread = await threadRepository.requestThread(
          event.wid, event.type, event.aid);
      yield RequestThreadSuccess(thread);
    } catch (error) {
      print(error);
      yield ThreadLoadError();
    }
  }

  Stream<ThreadState> _mapRequestAgentToState(RequestAgentEvent event) async* {
    yield ThreadLoading();
    try {
      final RequestThreadResult thread =
          await threadRepository.requestAgent(event.wid, event.type, event.aid);
      yield RequestAgentSuccess(thread);
    } catch (error) {
      print(error);
      yield ThreadLoadError();
    }
  }
}
