import 'dart:async';

import 'package:bytedesk_kefu/http/http_api_client.dart';
import 'package:bytedesk_kefu/model/oauth.dart';
import 'package:bytedesk_kefu/ui/chat/provider/chat_kf_provider.dart';
import 'package:bytedesk_kefu/util/constants.dart';
import 'package:bytedesk_kefu/util/utils.dart';
import 'package:flustars/flustars.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import 'model/user.dart';

class BytedeskKefu {
  //
  static const MethodChannel _channel = const MethodChannel('bytedesk_kefu');

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  // 自定义接口

  // 访客匿名登录接口
  static void anonymousLogin(String appkey, String subDomain) async {
    /// sp初始化
    await SpUtil.getInstance();
    // 首先检测是否是第一次，如果是第一此启动
    String username = SpUtil.getString(BDConstants.username);
    print('username:' + username);
    if (username.isEmpty) {
      // 则调用注册接口，否则调用登录接口
      User user = await HttpApiClient().registerAnonymous(subDomain);
      //
      String password = user.username;
      login(
          user.username, password, appkey, subDomain, BDConstants.ROLE_VISITOR);
    } else {
      //
      String password = username;
      login(username, password, appkey, subDomain, BDConstants.ROLE_VISITOR);
    }
  }

  static void login(String username, String password, String appkey,
      String subDomain, String role) async {
    //
    if (role == BDConstants.ROLE_ADMIN) {
      if (!username.contains("@")) {
        username = username + "@" + subDomain;
      }
    }
    await HttpApiClient().oauth(username, password);
    bool isLogin = SpUtil.getBool(BDConstants.isLogin);
    if (isLogin) {
      // 登录成功之后，建立长连接
      Utils.mqttConnect();
    }
  }

  // 技能组客服会话
  static void startWorkGroupChat(
      BuildContext context, String wid, String title) {
    Navigator.of(context).push(new MaterialPageRoute(builder: (context) {
      return new ChatKFProvider(
        wid: wid,
        aid: '',
        type: BDConstants.CHAT_TYPE_WORKGROUP,
        title: title,
      );
    }));
  }

  // 指定客服会话
  static void startAppointedChat(
      BuildContext context, String uid, String title) {
    Navigator.of(context).push(new MaterialPageRoute(builder: (context) {
      return new ChatKFProvider(
        wid: '',
        aid: uid,
        type: BDConstants.CHAT_TYPE_APPOINTED,
        title: title,
      );
    }));
  }
}
