import 'dart:convert';
import 'dart:io';

import 'package:bytedesk_kefu/model/model.dart';
import 'package:bytedesk_kefu/util/constants.dart';
import 'package:flustars/flustars.dart';
import 'package:http/http.dart' as http;

class HttpApiClient {
  static const uid = BDConstants.HELP_CATEGORY_UID;
  static const client = BDConstants.client;
  static String baseUrl = Platform.isAndroid
      ? BDConstants.httpBaseUrlAndroid
      : BDConstants.httpBaseUrliOS;

  final http.Client httpClient = http.Client();

  HttpApiClient();

  // 授权
  Future<OAuth> oauth(String username, String password) async {
    final oauthUrl = '$baseUrl/oauth/token';
    print("http api client: oauthUrl $oauthUrl");

    Map<String, String> headers = {
      "Authorization": "Basic Y2xpZW50OnNlY3JldA=="
    };
    Map<String, String> bodyMap = {
      "username": "$username",
      "password": "$password",
      "grant_type": "password",
      "scope": "all"
    };

    final oauthResponse =
        await this.httpClient.post(oauthUrl, headers: headers, body: bodyMap);
    // print('oauth result: $oauthResponse');
    // check the status code for the result
    int statusCode = oauthResponse.statusCode;
    print("statusCode $statusCode");

    // 200: 授权成功，否则授权失败
    final oauthJson = jsonDecode(oauthResponse.body);
    print("oauthJson $oauthJson");
    //
    // if (statusCode == 200) {
    SpUtil.putBool(BDConstants.isLogin, true);
    SpUtil.putString(BDConstants.accessToken, oauthJson['access_token']);
    // }
    //
    return OAuth.fromJson(statusCode, oauthJson);
  }

  // 验证码登录
  Future<OAuth> smsOAuth(String mobile, String code) async {
    //
    final oauthUrl = '$baseUrl/mobile/token';
    print("http api client: oauthUrl $oauthUrl");

    Map<String, String> headers = {
      "Authorization": "Basic Y2xpZW50OnNlY3JldA=="
    };
    Map<String, String> bodyMap = {
      "mobile": "$mobile",
      "code": "$code",
      "grant_type": "mobile",
      "scope": "all"
    };

    final oauthResponse =
        await this.httpClient.post(oauthUrl, headers: headers, body: bodyMap);
    print('oauth result: $oauthResponse');

    // check the status code for the result
    int statusCode = oauthResponse.statusCode;
    print("statusCode $statusCode");

    // 200: 授权成功，否则授权失败
    final oauthJson = jsonDecode(oauthResponse.body);
    print("oauthJson $oauthJson");
    // Preference.setAccessToken(oauthJson['access_token']);
    //
    if (statusCode == 200) {
      SpUtil.putBool(BDConstants.isLogin, true);
      SpUtil.putString(BDConstants.accessToken, oauthJson['access_token']);
    }

    return OAuth.fromJson(statusCode, oauthJson);
  }

  // 良师宝-手机号注册
  Future<JsonResult> register(String mobile, String password) async {
    Map<String, String> headers = {"Content-Type": "application/json"};

    var body = json.encode({
      "mobile": mobile,
      "password": password,
      "admin": false, // 学校端时，修改为true
      "client": "flutter"
    });

    final initUrl = '$baseUrl/visitors/api/v1/register/mobile';
    final initResponse =
        await this.httpClient.post(initUrl, headers: headers, body: body);

    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson =
        json.decode(utf8decoder.convert(initResponse.bodyBytes));
    // final responseJson = json.decode(initResponse.body);
    print("responseJson $responseJson");

    return JsonResult.fromJson(responseJson);
  }

  // 萝卜丝-访客端-注册匿名用户
  Future<User> registerAnonymous(String subDomain) async {
    //
    Map<String, String> headers = {"Content-Type": "application/json"};
    //
    final initUrl =
        '$baseUrl/visitor/api/username?subDomain=$subDomain&client=flutter';
    final initResponse = await this.httpClient.get(initUrl, headers: headers);

    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson =
        json.decode(utf8decoder.convert(initResponse.bodyBytes));
    print("responseJson $responseJson");
    //
    User user = User.fromJson(responseJson['data']);
    //
    SpUtil.putString(BDConstants.uid, user.uid);
    SpUtil.putString(BDConstants.username, user.username);
    SpUtil.putString(BDConstants.nickname, user.nickname);
    SpUtil.putString(BDConstants.avatar, user.avatar);
    SpUtil.putString(BDConstants.description, user.description);
    SpUtil.putString(BDConstants.subDomain, user.subDomain);
    SpUtil.putString(BDConstants.role, BDConstants.ROLE_VISITOR);
    // 解析用户资料
    return user;
  }

  // 注册自定义普通用户：用于IM,
  Future<User> registerUser(String username, String nickname, String password,
      String subDomain) async {
    //
    Map<String, String> headers = {"Content-Type": "application/json"};
    var body = json.encode({
      "username": username,
      "nickname": nickname,
      "password": password,
      "subDomain": subDomain,
      "client": "flutter"
    });

    final initUrl = '$baseUrl/visitor/api/register/user';
    final initResponse =
        await this.httpClient.post(initUrl, headers: headers, body: body);

    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson =
        json.decode(utf8decoder.convert(initResponse.bodyBytes));
    // final responseJson = json.decode(initResponse.body);
    print("responseJson $responseJson");
    // return JsonResult.fromJson(responseJson);
    //
    User user = User.fromJson(responseJson['data']);
    //
    SpUtil.putString(BDConstants.uid, user.uid);
    SpUtil.putString(BDConstants.nickname, user.nickname);
    SpUtil.putString(BDConstants.avatar, user.avatar);
    SpUtil.putString(BDConstants.description, user.description);
    // 解析用户资料
    return user;
  }

  // 修改密码
  Future<JsonResult> changePassword(String mobile, String password) async {
    Map<String, String> headers = {"Content-Type": "application/json"};

    var body = json
        .encode({"mobile": mobile, "password": password, "client": "flutter"});

    final initUrl = '$baseUrl/visitors/api/v1/change';
    final initResponse =
        await this.httpClient.post(initUrl, headers: headers, body: body);

    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson =
        json.decode(utf8decoder.convert(initResponse.bodyBytes));
    // final responseJson = json.decode(initResponse.body);
    print("responseJson $responseJson");

    return JsonResult.fromJson(responseJson);
  }

  Future<CodeResult> requestCode(String mobile) async {
    //
    Map<String, String> headers = {"Content-Type": "application/json"};
    //
    final initUrl =
        '$baseUrl/sms/api/send/liangshibao?mobile=$mobile&client=flutter';
    final initResponse = await this.httpClient.get(initUrl, headers: headers);
    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson =
        json.decode(utf8decoder.convert(initResponse.bodyBytes));
    // final responseJson = json.decode(initResponse.body);
    print("responseJson $responseJson");

    SpUtil.putBool(BDConstants.exist, responseJson['data']['exist']);
    SpUtil.putString(BDConstants.code, responseJson['data']['code']);

    return CodeResult.fromJson(responseJson);
  }

  /// 初始化
  Future<User> getProfile() async {
    //
    String accessToken = SpUtil.getString(BDConstants.accessToken);
    Map<String, String> headers = {
      HttpHeaders.contentTypeHeader: "application/json",
      HttpHeaders.authorizationHeader: "Bearer $accessToken"
    };
    final initUrl = '$baseUrl/api/user/profile?client=flutter';
    final initResponse = await this.httpClient.get(initUrl, headers: headers);
    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson =
        json.decode(utf8decoder.convert(initResponse.bodyBytes));
    print("responseJson $responseJson");
    //
    User user = User.fromJson(responseJson['data']);
    //
    SpUtil.putString(BDConstants.uid, user.uid);
    SpUtil.putString(BDConstants.nickname, user.nickname);
    SpUtil.putString(BDConstants.avatar, user.avatar);
    SpUtil.putString(BDConstants.description, user.description);
    // 解析用户资料
    return user;
  }

  // 更新头像
  Future<User> updateAvatar(String avatar) async {
    //
    String accessToken = SpUtil.getString(BDConstants.accessToken);
    Map<String, String> headers = {
      HttpHeaders.contentTypeHeader: "application/json",
      HttpHeaders.authorizationHeader: "Bearer $accessToken"
    };
    //
    var body = json.encode({"avatar": avatar, "client": "flutter"});
    //
    final initUrl = '$baseUrl/api/user/avatar';
    final initResponse =
        await this.httpClient.post(initUrl, headers: headers, body: body);
    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson =
        json.decode(utf8decoder.convert(initResponse.bodyBytes));
    print("responseJson $responseJson");
    // TODO: 更新本地数据

    return User.fromJson(responseJson);
  }

  // 更新昵称
  Future<User> updateNickname(String nickname) async {
    //
    String accessToken = SpUtil.getString(BDConstants.accessToken);
    Map<String, String> headers = {
      HttpHeaders.contentTypeHeader: "application/json",
      HttpHeaders.authorizationHeader: "Bearer $accessToken"
    };
    //
    var body = json.encode({"nickname": nickname, "client": "flutter"});
    //
    final initUrl = '$baseUrl/api/user/nickname';
    final initResponse =
        await this.httpClient.post(initUrl, headers: headers, body: body);
    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson =
        json.decode(utf8decoder.convert(initResponse.bodyBytes));
    print("responseJson $responseJson");
    // TODO: 更新本地数据

    return User.fromJson(responseJson);
  }

  // 更新个性签名
  Future<User> updateDescription(String description) async {
    //
    String accessToken = SpUtil.getString(BDConstants.accessToken);
    Map<String, String> headers = {
      HttpHeaders.contentTypeHeader: "application/json",
      HttpHeaders.authorizationHeader: "Bearer $accessToken"
    };
    //
    var body = json.encode({"description": description, "client": "flutter"});
    //
    final initUrl = '$baseUrl/api/user/description';
    final initResponse =
        await this.httpClient.post(initUrl, headers: headers, body: body);
    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson =
        json.decode(utf8decoder.convert(initResponse.bodyBytes));
    print("responseJson $responseJson");
    // TODO: 更新本地数据

    return User.fromJson(responseJson);
  }

  // 加载会话列表
  // TODO: 分页拉取
  Future<List<Thread>> getThreads() async {
    String accessToken = SpUtil.getString(BDConstants.accessToken);
    //
    Map<String, String> headers = {
      // "Accept":  "application/json",
      HttpHeaders.contentTypeHeader: "application/json",
      HttpHeaders.authorizationHeader: "Bearer $accessToken"
    };
    final threadUrl = '$baseUrl/api/thread/get?client=flutter';
    // print("thread Url $threadUrl");
    final initResponse = await this.httpClient.get(threadUrl, headers: headers);

    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson =
        json.decode(utf8decoder.convert(initResponse.bodyBytes));
    print("responseJson $responseJson");

    List<Thread> threadList = [];

    List<Thread> agentThreadList =
        (responseJson['data']['agentThreads'] as List<dynamic>)
            .map((item) => Thread.fromWorkGroupJson(item))
            .toList();
    threadList.addAll(agentThreadList);

    List<Thread> contactThreadList =
        (responseJson['data']['contactThreads'] as List<dynamic>)
            .map((item) => Thread.fromContactJson(item))
            .toList();
    threadList.addAll(contactThreadList);

    List<Thread> groupThreadList =
        (responseJson['data']['groupThreads'] as List<dynamic>)
            .map((item) => Thread.fromGroupJson(item))
            .toList();
    threadList.addAll(groupThreadList);

    return threadList;
  }

  // 加载访客会话列表
  // 分页拉取
  Future<List<Thread>> getVisitorThreads(int page, int size) async {
    //
    String accessToken = SpUtil.getString(BDConstants.accessToken);
    Map<String, String> headers = {
      HttpHeaders.contentTypeHeader: "application/json",
      HttpHeaders.authorizationHeader: "Bearer $accessToken"
    };
    final threadUrl = '$baseUrl/api/thread/visitor/history?client=flutter';
    final initResponse = await this.httpClient.get(threadUrl, headers: headers);
    //
    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson =
        json.decode(utf8decoder.convert(initResponse.bodyBytes));
    print("responseJson $responseJson");

    List<Thread> threadList = (responseJson['data']["content"] as List<dynamic>)
        .map((item) => Thread.fromWorkGroupJson(item))
        .toList();
    return threadList;
  }

  // 请求客服会话
  Future<RequestThreadResult> requestThread(
      String wid, String type, String aid) async {
    String accessToken = SpUtil.getString(BDConstants.accessToken);
    print('access token:' + accessToken);
    //
    Map<String, String> headers = {
      HttpHeaders.contentTypeHeader: "application/json",
      HttpHeaders.authorizationHeader: "Bearer $accessToken"
    };
    final threadUrl =
        '$baseUrl/api/thread/request?wId=$wid&type=$type&aId=$aid&client=flutter';
    print("request thread Url $threadUrl");
    final initResponse = await this.httpClient.get(threadUrl, headers: headers);

    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson =
        json.decode(utf8decoder.convert(initResponse.bodyBytes));
    print("responseJson $responseJson");

    return RequestThreadResult.fromJson(responseJson);
  }

  // 请求人工客服，不管此工作组是否设置为默认机器人，只要有人工客服在线，则可以直接对接人工
  Future<RequestThreadResult> requestAgent(
      String wid, String type, String aid) async {
    String accessToken = SpUtil.getString(BDConstants.accessToken);
    //
    Map<String, String> headers = {
      HttpHeaders.contentTypeHeader: "application/json",
      HttpHeaders.authorizationHeader: "Bearer $accessToken"
    };
    final threadUrl =
        '$baseUrl/api/thread/request/agent?wId=$wid&type=$type&aId=$aid&client=flutter';
    print("request agent Url $threadUrl");
    final initResponse = await this.httpClient.get(threadUrl, headers: headers);

    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson =
        json.decode(utf8decoder.convert(initResponse.bodyBytes));
    print("responseJson $responseJson");

    return RequestThreadResult.fromJson(responseJson);
  }

  // 常见问题分类
  Future<List<HelpCategory>> getHelpSupportCategories() async {
    //
    final categoriesUrl =
        '$baseUrl/visitor/api/category/support?uid=$uid&client=$client';
    print("categories Url $categoriesUrl");
    final initResponse = await this.httpClient.get(categoriesUrl);
    //
    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson =
        json.decode(utf8decoder.convert(initResponse.bodyBytes));
    //
    List<HelpCategory> categories = (responseJson['data'] as List<dynamic>)
        .map((item) => HelpCategory.fromJson(item))
        .toList();

    return categories;
  }

  // 常见问题分类-所属文章
  Future<List<HelpArticle>> getHelpSupportArticles(int categoryId) async {
    //
    final categoriesUrl =
        '$baseUrl/visitor/api/category/articles?categoryId=$categoryId&client=$client';
    print("categories Url $categoriesUrl");
    final initResponse = await this.httpClient.get(categoriesUrl);
    //
    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson =
        json.decode(utf8decoder.convert(initResponse.bodyBytes));
    //
    List<HelpArticle> articles = (responseJson['data'] as List<dynamic>)
        .map((item) => HelpArticle.fromJson(item))
        .toList();
    //
    return articles;
  }

  //
  Future<List<HelpCategory>> getHelpFeedbackCategories() async {
    //
    final categoriesUrl =
        '$baseUrl/visitor/api/category/feedback?uid=$uid&client=$client';
    print("categories Url $categoriesUrl");
    final initResponse = await this.httpClient.get(categoriesUrl);
    //
    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson =
        json.decode(utf8decoder.convert(initResponse.bodyBytes));
    //
    List<HelpCategory> categories = (responseJson['data'] as List<dynamic>)
        .map((item) => HelpCategory.fromJson(item))
        .toList();

    return categories;
  }

  // https://pub.dev/documentation/http/latest/http/MultipartRequest-class.html
  Future<String> upload(String filePath) async {
    String fileName = filePath.split("/").last;
    String username = SpUtil.getString(BDConstants.uid);

    final uploadUrl = '$baseUrl/visitor/api/upload/image';
    print("upload Url $uploadUrl");

    var uri = Uri.parse(uploadUrl);
    var request = http.MultipartRequest('POST', uri)
      ..fields['file_name'] = fileName
      ..fields['username'] = username
      ..files.add(await http.MultipartFile.fromPath('file', filePath));

    http.Response response =
        await http.Response.fromStream(await request.send());
    // print("Result: ${response.body}");

    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson = json.decode(utf8decoder.convert(response.bodyBytes));
    print("responseJson $responseJson");

    String url = responseJson['data'];
    print('url:' + url);
    return url;
  }

  //
  Future<RequestAnswerResult> queryAnswer(String tid, String aid) async {
    //
    String accessToken = SpUtil.getString(BDConstants.accessToken);
    Map<String, String> headers = {
      // "Accept":  "application/json",
      HttpHeaders.contentTypeHeader: "application/json",
      HttpHeaders.authorizationHeader: "Bearer $accessToken"
    };
    //
    final queryAnswerUrl =
        '$baseUrl/api/answer/query?tid=$tid&aid=$aid&client=$client';
    print("query Url $queryAnswerUrl");
    final initResponse =
        await this.httpClient.get(queryAnswerUrl, headers: headers);
    //
    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson =
        json.decode(utf8decoder.convert(initResponse.bodyBytes));
    //
    return RequestAnswerResult.fromJson(responseJson);
  }

  //
  Future<RequestAnswerResult> messageAnswer(
      String type, String wid, String aid, String content) async {
    //
    String accessToken = SpUtil.getString(BDConstants.accessToken);
    Map<String, String> headers = {
      HttpHeaders.contentTypeHeader: "application/json",
      HttpHeaders.authorizationHeader: "Bearer $accessToken"
    };
    //
    final messageAnswerUrl =
        '$baseUrl/api/v2/answer/message?type=$type&wid=$wid&aid=$aid&content=$content&client=$client';
    print("message Url $messageAnswerUrl");
    final initResponse =
        await this.httpClient.get(messageAnswerUrl, headers: headers);
    //
    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson =
        json.decode(utf8decoder.convert(initResponse.bodyBytes));
    print("messageAnswer responseJson $responseJson");
    //
    return RequestAnswerResult.fromJson(responseJson);
  }

  //
  Future<RequestAnswerResult> rateAnswer(
      String aid, String mid, bool rate) async {
    //
    String accessToken = SpUtil.getString(BDConstants.accessToken);
    Map<String, String> headers = {
      HttpHeaders.contentTypeHeader: "application/json",
      HttpHeaders.authorizationHeader: "Bearer $accessToken"
    };
    //
    final rateAnswerUrl =
        '$baseUrl/api/answer/rate?aid=$aid&mid=$mid&rate=$rate&client=$client';
    print("rate Url $rateAnswerUrl");
    final initResponse =
        await this.httpClient.get(rateAnswerUrl, headers: headers);
    //
    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson =
        json.decode(utf8decoder.convert(initResponse.bodyBytes));
    //
    return RequestAnswerResult.fromRateJson(responseJson);
  }

  //
  Future<void> logout() async {
    String accessToken = SpUtil.getString(BDConstants.accessToken);
    Map<String, String> headers = {"Content-Type": "application/json"};

    var body = json.encode({"client": "flutter"});

    final initUrl = '$baseUrl/api/user/logout?access_token=$accessToken';
    final initResponse =
        await this.httpClient.post(initUrl, headers: headers, body: body);

    final responseJson = json.decode(initResponse.body);
    print("responseJson $responseJson");
    //
    // Preference.clearAccessToken();
  }
}
