import 'package:bytedesk_kefu/model/answer.dart';
import 'package:bytedesk_kefu/model/thread.dart';
import 'package:bytedesk_kefu/model/user.dart';
import 'package:bytedesk_kefu/util/constants.dart';
import 'package:equatable/equatable.dart';

class Message extends Equatable {
  //
  String mid;
  String content;
  String imageUrl;
  String avatar;
  String type;
  String topic;
  String timestamp;
  String status;
  int isSend;
  //
  Thread thread;
  User user;
  //
  List<Answer> answers;

  Message(
      {this.mid,
      this.content,
      this.imageUrl,
      this.avatar,
      this.type,
      this.topic,
      this.timestamp,
      this.isSend,
      this.thread,
      this.user,
      this.status,
      this.answers})
      : super();

  //
  static Message fromJsonThread(dynamic json) {
    //
    String content = json['content'];
    List<Answer> robotQaList = [];
    if (json['type'] == BDConstants.MESSAGE_TYPE_ROBOT) {
      robotQaList = (json['answers'] as List<dynamic>)
          .map((item) => Answer.fromJson(item))
          .toList();
      for (var i = 0; i < robotQaList.length; i++) {
        Answer answer = robotQaList[i];
        content += '\n\n' + answer.aid + ':' + answer.question;
      }
    }
    return Message(
        mid: json['mid'],
        content: content,
        imageUrl: json['imageUrl'],
        avatar: json['user']['avatar'],
        type: json['type'],
        timestamp: json['createdAt'],
        status: 'stored',
        isSend: 0,
        thread: Thread.fromVisitorJson(json['thread']),
        user: User.fromJson(json['user']),
        answers: robotQaList);
  }

  static Message fromJson(dynamic json) {
    return Message(
        mid: json['mid'],
        content: json['content'],
        imageUrl: json['imageUrl'],
        avatar: json['user']['avatar'],
        type: json['type'],
        timestamp: json['timestamp'],
        answers: []);
  }

  @override
  List<Object> get props => [mid, content, avatar, type, isSend];

  // Convert a Message into a Map. The keys must correspond to the names of the
  // columns in the database.
  Map<String, dynamic> toMap() {
    return {
      'mid': mid,
      'content': content,
      'imageUrl': imageUrl,
      'avatar': avatar,
      'type': type,
      'topic': thread.topic,
      'status': status,
      'timestamp': timestamp,
      'isSend': isSend
    };
  }

  Message.fromMap(Map<String, dynamic> map) {
    mid = map['mid'];
    content = map['content'];
    imageUrl = map['imageUrl'];
    avatar = map['avatar'];
    type = map['type'];
    topic = map['topic'];
    status = map['status'];
    timestamp = map['timestamp'];
    isSend = map['isSend'];
  }
}
