import 'dart:convert';

import 'package:bytedesk_kefu/model/message.dart';
import 'package:bytedesk_kefu/model/thread.dart';
import 'package:bytedesk_kefu/model/user.dart';
import 'package:bytedesk_kefu/mqtt/lib/mqtt_client.dart';
import 'package:bytedesk_kefu/mqtt/lib/mqtt_server_client.dart';
import 'package:bytedesk_kefu/util/constants.dart';
import 'package:bytedesk_kefu/model/messageProvider.dart';
import 'package:bytedesk_kefu/util/events.dart';
import 'package:bytedesk_kefu/util/uuid.dart';
import 'package:flustars/flustars.dart';
import 'package:bytedesk_kefu/protobuf/message.pb.dart' as protomsg;
import 'package:bytedesk_kefu/protobuf/thread.pb.dart' as protothread;
import 'package:bytedesk_kefu/protobuf/user.pb.dart' as protouser;

class BdMqtt {
  MqttClient mqttClient;
  String clientId;
  int keepAlivePeriod = 20;
  // final key = Key.fromUtf8('16BytesLengthKey');
  // final iv = IV.fromUtf8('A-16-Byte-String');
  MessageProvider messageProvider = new MessageProvider();
  // EventBus eventBus = EventBus();
  List<String> midList = [];

  static final BdMqtt _singleton = BdMqtt._internal();

  factory BdMqtt() {
    return _singleton;
  }

  BdMqtt._internal() {
    _connect();
  }

  // BdMqtt()
  //   // 建立连接
  //   _connect();
  // }

  void _connect() async {
    //
    reconnect();
  }

  void reconnect() async {
    // eventbus发送广播，连接中...
    eventBus.fire(ConnectionEventBus(BDConstants.USER_STATUS_CONNECTING));
    //
    String currentUid = SpUtil.getString(BDConstants.uid);
    String client = BDConstants.clientMqtt;
    clientId = "$currentUid/$client";

    // mqttClient = MqttClient.withPort(url, clientId, port);
    // mqttClient = MqttClient(BDConstants.mqttUrl, '');
    mqttClient = MqttServerClient(BDConstants.mqttUrl, '');
    // 启用3.1.1版本协议，否则clientId限制最大长度为23
    mqttClient.setProtocolV311();
    mqttClient.port = BDConstants.mqttPort;

    /// Set logging on if needed, defaults to off
    mqttClient.logging(on: BDConstants.isDebug);

    /// If you intend to use a keep alive value in your connect message that is not the default(60s)
    /// you must set it here
    mqttClient.keepAlivePeriod = keepAlivePeriod;
    mqttClient.autoReconnect = true;
    // mqttClient.onAutoReconnect = _onAutoReconnect;
    mqttClient.onDisconnected = _onDisconnected;
    mqttClient.onConnected = _onConnected;
    mqttClient.onSubscribed = _onSubscribed;
    mqttClient.onUnsubscribed = _onUnSubscribed;
    mqttClient.onSubscribeFail = _onSubscribeFailed;

    /// Set a ping received callback if needed, called whenever a ping response(pong) is received from the broker.
    mqttClient.pongCallback = _onPong;

    /// Create a connection message to use or use the default one. The default one sets the
    /// client identifier, any supplied username/password, the default keepalive interval(60s)
    /// and clean session, an example of a specific one below.

    final MqttConnectMessage connMess = MqttConnectMessage()
        .withClientIdentifier(clientId)
        .authenticateAs('username', 'password') // TODO: 服务器暂时不需要auth，随便填写
        .keepAliveFor(
            keepAlivePeriod); // Must agree with the keep alive set above or not set
    // 取消客户端设置，直接在服务器端统一内容格式推送
    // .withWillTopic('protobuf/lastWill/mqtt') // If you set this you must set a will message
    // .withWillMessage('My Will message')
    // .startClean() // Non persistent session for testing
    // .withWillQos(MqttQos.atLeastOnce);
    print('mqttClient connecting....');
    mqttClient.connectionMessage = connMess;

    /// Connect the client, any errors here are communicated by raising of the appropriate exception. Note
    /// in some circumstances the broker will just disconnect us, see the spec about this, we however eill
    /// never send malformed messages.
    try {
      await mqttClient.connect();
    } on Exception catch (e) {
      print('mqttClient exception - $e');
      mqttClient.disconnect();
    }

    /// Check we are connected
    if (mqttClient.connectionStatus.state == MqttConnectionState.connected) {
      print('mqttClient connected');
    } else {
      /// Use status here rather than state if you also want the broker return code.
      print(
          'ERROR mqttClient connection failed - disconnecting, status is ${mqttClient.connectionStatus}');
      mqttClient.disconnect();
      // exit(-1);
    }

    /// The client has a change notifier object(see the Observable class) which we then listen to to get
    /// notifications of published updates to each subscribed topic.
    // mqttClient.updates.listen((List<MqttReceivedMessage<MqttMessage>> c) {
    //   final MqttPublishMessage recMess = c[0].payload;
    //   final String pt =
    //       MqttPublishPayload.bytesToStringAsString(recMess.payload.message);
    //   // print('Change notification:: topic is <${c[0].topic}>, payload is <-- $pt -->');
    // });

    /// 收到的消息
    // if (mqttClient != null && mqttClient.published != null) {
    //
    mqttClient.published.listen((MqttPublishMessage messageBinary) {
      // print('Published notification:: topic is ${messageBinary.variableHeader.topicName}, with Qos ${messageBinary.header.qos}');
      //
      protomsg.Message messageProto =
          protomsg.Message.fromBuffer(messageBinary.payload.message);
      // FIXME: 自己发送的消息显示两条？此处根据mid去个重
      var mid = messageProto.mid;
      if (midList.contains(mid)) {
        return;
      }
      midList.add(mid);
      //
      var uid = messageProto.user.uid;
      var nickname = messageProto.user.nickname;
      var avatar = messageProto.user.avatar;
      //
      var content = '';
      var type = messageProto.type;
      var timestamp = messageProto.timestamp;
      var client = messageProto.client;
      print('receive type:' + type + ' client:' + client + ' mid:' + mid);
      // 非会话消息，如：会议通知等, 另行处理
      if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_NOTICE) {
        // TODO: 待处理
        return;
      }
      //
      User user = new User(uid: uid, nickname: nickname, avatar: avatar);
      //
      Thread thread = new Thread(
          tid: messageProto.thread.tid,
          type: messageProto.thread.type,
          // uid: '',
          nickname: messageProto.thread.nickname,
          avatar: messageProto.thread.avatar,
          content: messageProto.thread.content,
          timestamp: messageProto.thread.timestamp,
          unreadCount: messageProto.thread.unreadCount,
          topic: messageProto.thread.topic,
          client: client);
      Message message = new Message(
          mid: mid,
          content: content,
          imageUrl: content,
          avatar: avatar,
          type: type,
          timestamp: timestamp,
          status: 'stored',
          isSend: uid == currentUid ? 1 : 0,
          thread: thread,
          user: user);
      // 是否发送消息回执
      var autoReply = false;
      var sendReceipt = false;
      // var webRTCVideoInvite = false;
      // var webRTCAudioInvite = false;
      switch (type) {
        case BDConstants.MESSAGE_TYPE_TEXT:
          {
            //
            autoReply = true;
            sendReceipt = true;
            // TODO: 判断是否加密，暂时不需要
            message.content = messageProto.text.content;
            break;
          }
        case BDConstants.MESSAGE_TYPE_IMAGE:
          {
            //
            autoReply = true;
            sendReceipt = true;
            message.imageUrl = messageProto.image.imageUrl;
            break;
          }
        case BDConstants.MESSAGE_TYPE_VOICE:
          {
            //
            autoReply = true;
            sendReceipt = true;
            break;
          }
        case BDConstants.MESSAGE_TYPE_FILE:
          {
            //
            autoReply = true;
            sendReceipt = true;
            break;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_THREAD:
          {
            message.content = messageProto.text.content;
            break;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_THREAD_REENTRY:
        case BDConstants.MESSAGE_TYPE_COMMODITY:
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_CONNECT:
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_DISCONNECT:
          {
            message.content = messageProto.text.content;
            break;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_AGENT_CLOSE:
          {
            message.content = messageProto.text.content;
            break;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_VISITOR_CLOSE:
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_AUTO_CLOSE:
          {
            message.content = messageProto.text.content;
            break;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_PREVIEW:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_RECEIPT:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_RECALL:
          {
            //
            var midR = messageProto.receipt.mid;
            var statusR = messageProto.receipt.status;
            messageProvider.update(midR, statusR);
            // 通知界面更新聊天记录状态
            eventBus.fire(ReceiveMessageReceiptEventBus(midR, statusR));
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_TRANSFER:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_TRANSFER_ACCEPT:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_TRANSFER_REJECT:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_INVITE:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_INVITE_ACCEPT:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_INVITE_REJECT:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_INVITE_RATE:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_RATE_RESULT:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_INVITE_VIDEO:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_INVITE_AUDIO:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_CANCEL:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_OFFER_VIDEO:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_OFFER_AUDIO:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_ANSWER:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_CANDIDATE:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_ACCEPT:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_REJECT:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_READY:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_BUSY:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_CLOSE:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_CREATE:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_UPDATE:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_ANNOUNCEMENT:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_INVITE:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_INVITE_ACCEPT:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_INVITE_REJECT:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_APPLY:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_APPLY_APPROVE:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_APPLY_DENY:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_KICK:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_MUTE:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_TRANSFER:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_TRANSFER_ACCEPT:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_TRANSFER_REJECT:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_WITHDRAW:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_DISMISS:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_VIDEO:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_SHORT_VIDEO:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_LOCATION:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_LINK:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_EVENT:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_CUSTOM:
          {
            //
            return;
          }
        case BDConstants.MESSAGE_TYPE_RED_PACKET:
          {
            //
            return;
          }
        default:
          print('other message type:' + type);
      }

      // 客服端使用
      if (autoReply) {
        //
      }

      //
      if (sendReceipt && message.isSend == 0) {
        // 发送送达回执
        sendReceiptReceivedMessage(mid, thread);
      }
      //
      // final encrypter = Encrypter(AES(key, mode: AESMode.cbc));
      // final decrypted = encrypter.decrypt64(messageProto.text.content, iv: iv);
      // print('content: ' + decrypted);
      // 插入本地数据库
      messageProvider.insert(message);
      // 通知界面显示聊天记录
      eventBus.fire(ReceiveMessageEventBus(message));
    });
    //
    // } else {
    //   print('mqttClient.published is null');
    // }
  }

  void subscribe(String topic) {
    // print('Subscribing to the hello topic');
    mqttClient.subscribe(topic, MqttQos.exactlyOnce);
  }

  void unsubscribe(String topic) {
    mqttClient.unsubscribe(topic);
  }

  void sendTextMessage(String content, Thread currentThread) {
    publish(content, null, BDConstants.MESSAGE_TYPE_TEXT, currentThread);
  }

  void sendImageMessage(String content, Thread currentThread) {
    publish(content, null, BDConstants.MESSAGE_TYPE_IMAGE, currentThread);
  }

  void sendReceiptReceivedMessage(String mid, Thread currentThread) {
    sendReceiptMessage(mid, BDConstants.MESSAGE_STATUS_RECEIVED, currentThread);
  }

  void sendReceiptReadMessage(String mid, Thread currentThread) {
    sendReceiptMessage(mid, BDConstants.MESSAGE_STATUS_READ, currentThread);
  }

  void sendReceiptMessage(String mid, String status, Thread currentThread) {
    protomsg.Receipt receipt = new protomsg.Receipt();
    receipt.mid = mid;
    receipt.status = status;
    publish('', receipt, BDConstants.MESSAGE_TYPE_NOTIFICATION_RECEIPT,
        currentThread);
  }

  void publish(String content, protomsg.Receipt receipt, String type,
      Thread currentThread) {
    // https://pub.dev/packages/encrypt#aes
    // final encrypter = Encrypter(AES(key, mode: AESMode.cbc));
    // final encrypted = encrypter.encrypt(content, iv: iv);
    // print(encrypted.base64);
    // final decrypted = encrypter.decrypt(encrypted, iv: iv);
    // print(decrypted);

    // thread
    protothread.Thread thread = new protothread.Thread();
    thread.tid = currentThread.tid;
    thread.type = currentThread.type;
    thread.topic = currentThread.topic;
    thread.nickname = currentThread.nickname;
    thread.avatar = currentThread.avatar;
    thread.timestamp = '';
    //CommonUtils.formatedDateNow(); // 没有必要填写，服务器端会填充
    thread.unreadCount = 0;
    var extra = {'top': false, 'undisturb': false};
    thread.extra = jsonEncode(extra);
    // user
    protouser.User user = new protouser.User();
    user.uid = SpUtil.getString(BDConstants.uid);
    user.nickname = SpUtil.getString(BDConstants.nickname);
    user.avatar = SpUtil.getString(BDConstants.avatar);
    // msg
    protomsg.Message message = new protomsg.Message();
    message.mid = Uuid.generateV4();
    message.type = type;
    message.timestamp = '';
    // CommonUtils.formatedDateNow(); // 没有必要填写，服务器端会填充
    message.client = BDConstants.clientMqtt;
    message.version = '1';
    message.encrypted = false;

    // 发送protobuf
    if (type == BDConstants.MESSAGE_TYPE_TEXT) {
      protomsg.Text text = new protomsg.Text();
      text.content = content;
      message.text = text;
      //
      thread.content = content;
    } else if (type == BDConstants.MESSAGE_TYPE_IMAGE) {
      protomsg.Image image = new protomsg.Image();
      image.imageUrl = content;
      message.image = image;
      //
      thread.content = '[图片]';
    } else if (type == BDConstants.MESSAGE_TYPE_VOICE) {
      protomsg.Voice voice = new protomsg.Voice();
      voice.voiceUrl = content;
      message.voice = voice;
      //
      thread.content = '[语音]';
    } else if (type == BDConstants.MESSAGE_TYPE_FILE) {
      protomsg.File file = new protomsg.File();
      file.fileUrl = content;
      message.file = file;
      //
      thread.content = '[文件]';
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_PREVIEW) {
      protomsg.Preview preview = new protomsg.Preview();
      preview.content = content;
      message.preview = preview;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_RECEIPT) {
      // 发送消息回执
      message.receipt = receipt;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_RECALL) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_TRANSFER) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_TRANSFER_ACCEPT) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_TRANSFER_REJECT) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_INVITE) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_INVITE_ACCEPT) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_INVITE_REJECT) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_INVITE_RATE) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_RATE_RESULT) {
      //
      return;
    } else if (type ==
        BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_INVITE_VIDEO) {
      //
      return;
    } else if (type ==
        BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_INVITE_AUDIO) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_CANCEL) {
      //
      return;
    } else if (type ==
        BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_OFFER_VIDEO) {
      //
      return;
    } else if (type ==
        BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_OFFER_AUDIO) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_ANSWER) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_CANDIDATE) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_ACCEPT) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_REJECT) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_READY) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_BUSY) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_WEBRTC_CLOSE) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_CREATE) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_UPDATE) {
      //
      return;
    } else if (type ==
        BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_ANNOUNCEMENT) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_INVITE) {
      //
      return;
    } else if (type ==
        BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_INVITE_ACCEPT) {
      //
      return;
    } else if (type ==
        BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_INVITE_REJECT) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_APPLY) {
      //
      return;
    } else if (type ==
        BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_APPLY_APPROVE) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_APPLY_DENY) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_KICK) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_MUTE) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_TRANSFER) {
      //
      return;
    } else if (type ==
        BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_TRANSFER_ACCEPT) {
      //
      return;
    } else if (type ==
        BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_TRANSFER_REJECT) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_WITHDRAW) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_NOTIFICATION_GROUP_DISMISS) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_VIDEO) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_SHORT_VIDEO) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_LOCATION) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_LINK) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_EVENT) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_CUSTOM) {
      //
      return;
    } else if (type == BDConstants.MESSAGE_TYPE_RED_PACKET) {
      //
      return;
    } else {
      // TODO: 其他类型消息
      print('other type:' + type);
    }
    //
    message.user = user;
    message.thread = thread;
    //
    final MqttClientPayloadBuilder builder = MqttClientPayloadBuilder();
    // 注意：此函数为自行添加，原先库中没有
    builder.addProtobuf(message.writeToBuffer());
    mqttClient.publishMessage(
        currentThread.topic, MqttQos.exactlyOnce, builder.payload);
  }

  // 断开长连接
  void disconnect() {
    mqttClient.disconnect();
  }

  /// The subscribed callback
  void _onSubscribed(String topic) {
    // print('Subscription confirmed for topic $topic');
  }

  /// The unsubscribed callback
  void _onUnSubscribed(String topic) {
    // print('UnSubscription confirmed for topic $topic');
  }

  /// The subscribed callback
  void _onSubscribeFailed(String topic) {
    // print('Subscribe Failed confirmed for topic $topic');
  }

  /// The unsolicited disconnect callback
  void _onDisconnected() {
    // print('OnDisconnected client callback - Client disconnection');
    // eventbus发广播，通知长连接断开
    eventBus.fire(ConnectionEventBus(BDConstants.USER_STATUS_DISCONNECTED));
    // if (mqttClient.connectionStatus.returnCode == MqttConnectReturnCode.solicited) {
    //   print('OnDisconnected callback is solicited, this is correct');
    // }
    // 延时10s执行重连
    Future.delayed(Duration(seconds: 10), () {
      // print('start reconnecting');
      // reconnect();
    });
  }

  /// The unsolicited disconnect callback
  void _onAutoReconnect() {
    // print('EXAMPLE::onAutoReconnect client callback - Client auto reconnection sequence will start');
    // connect();
  }

  /// The successful connect callback
  void _onConnected() {
    // print('OnConnected client callback - Client connection was sucessful');
    // TODO: eventbus发广播，通知长连接建立
    eventBus.fire(ConnectionEventBus(BDConstants.USER_STATUS_CONNECTED));
  }

  /// Pong callback
  void _onPong() {
    // print('Ping response client callback invoked');
  }
}
