import 'package:bytedesk_kefu/blocs/message_bloc/bloc.dart';
import 'package:bytedesk_kefu/blocs/thread_bloc/bloc.dart';
import 'package:bytedesk_kefu/model/messageProvider.dart';
import 'package:bytedesk_kefu/model/model.dart';
import 'package:bytedesk_kefu/mqtt/mqtt.dart';
import 'package:bytedesk_kefu/ui/chat/widget/message_widget.dart';
import 'package:bytedesk_kefu/util/events.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:image_picker/image_picker.dart';

class ChatKFPage extends StatefulWidget {
  //
  final String wid;
  final String aid;
  final String type;
  final String title;
  //
  ChatKFPage({Key key, this.wid, this.aid, this.type, this.title})
      : super(key: key);

  @override
  _ChatKFPageState createState() => _ChatKFPageState();
}

class _ChatKFPageState extends State<ChatKFPage>
    with
        AutomaticKeepAliveClientMixin<ChatKFPage>,
        TickerProviderStateMixin,
        WidgetsBindingObserver {
  //
  MessageProvider messageProvider = new MessageProvider();
  final TextEditingController _textController = new TextEditingController();
  final List<MessageWidget> _messages = <MessageWidget>[];
  final ImagePicker picker = ImagePicker();
  final BdMqtt _bdMqtt = new BdMqtt();
  Thread _currentThread;
  bool isRobot;
  //
  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance.addObserver(this);
    _listener();
    setState(() {
      isRobot = false;
    });
  }

  //
  @override
  Widget build(BuildContext context) {
    super.build(context);
    //
    return Scaffold(
        appBar: AppBar(
          title: Text(widget.title),
          elevation: 0,
          // centerTitle: widget.title,
          // actionName: '常见问题',
          // onPressed: () {
          //   // FIXME: 当前跳转至-良师宝-官方常见问题，后期改为商家常见问题？
          //   // Navigator.of(context)
          //   //     .push(new MaterialPageRoute(builder: (context) {
          //   //   return new HelpProvider();
          //   // }));
          // },
        ),
        body: MultiBlocListener(
          listeners: [
            BlocListener<ThreadBloc, ThreadState>(
              listener: (context, state) {
                if (state is RequestThreadSuccess) {
                  setState(() {
                    isRobot = false;
                    _currentThread = state.threadResult.msg.thread;
                  });
                  _getMessages();
                  _appendMessage(state.threadResult.msg);
                  if (state.threadResult.statusCode == 200 ||
                      state.threadResult.statusCode == 201) {
                    print('创建新会话');
                  } else if (state.threadResult.statusCode == 202) {
                    print('提示排队中');
                  } else if (state.threadResult.statusCode == 203) {
                    print('当前非工作时间，请自助查询或留言');
                  } else if (state.threadResult.statusCode == 204) {
                    print('当前无客服在线，请自助查询或留言');
                  } else if (state.threadResult.statusCode == 205) {
                    print('咨询前问卷');
                  } else if (state.threadResult.statusCode == 206) {
                    print('返回机器人初始欢迎语 + 欢迎问题列表');
                    setState(() {
                      isRobot = true;
                    });
                  } else {
                    Fluttertoast.showToast(msg: "请求会话失败");
                  }
                }
              },
            ),
            BlocListener<MessageBloc, MessageState>(
              listener: (context, state) {
                print('message state change');
                if (state is ReceiveMessageState) {
                  print('receive message:' + state.message.content);
                } else if (state is UploadImageSuccess) {
                  _bdMqtt.sendImageMessage(state.url, _currentThread);
                } else if (state is QueryAnswerSuccess) {
                } else if (state is MessageAnswerSuccess) {
                  _appendMessage(state.query);
                  if (state.query.content.contains('人工')) {
                    BlocProvider.of<ThreadBloc>(context)
                      ..add(RequestAgentEvent(
                          wid: widget.wid, aid: widget.aid, type: widget.type));
                  } else {
                    _appendMessage(state.answer);
                  }
                } else if (state is RateAnswerSuccess) {}
              },
            ),
          ],
          child: new Container(
            alignment: Alignment.bottomCenter,
            child: new Column(
              children: <Widget>[
                new Flexible(
                  child: new ListView.builder(
                    padding: new EdgeInsets.all(8.0),
                    reverse: true,
                    itemBuilder: (_, int index) => _messages[index],
                    itemCount: _messages.length,
                  ),
                ),
                new Divider(
                  height: 1.0,
                ),
                new Container(
                  decoration: new BoxDecoration(
                    color: Theme.of(context).cardColor,
                  ),
                  child: _textComposerWidget(),
                ),
              ],
            ),
          ),
        ));
  }

  //
  Widget _textComposerWidget() {
    return new IconTheme(
      data: new IconThemeData(color: Colors.blue),
      child: new Container(
        margin: const EdgeInsets.symmetric(horizontal: 8.0),
        child: new Row(
          children: <Widget>[
            new Container(
              // margin: const EdgeInsets.symmetric(horizontal: 4.0),
              child: new IconButton(
                icon: new Icon(Icons.add),
                onPressed: () {
                  _getImage();
                },
              ),
            ),
            new Flexible(
              child: new TextField(
                decoration: new InputDecoration.collapsed(hintText: "输入内容..."),
                controller: _textController,
                onSubmitted: _handleSubmitted,
              ),
            ),
            new Container(
              margin: const EdgeInsets.symmetric(horizontal: 4.0),
              child: new IconButton(
                icon: new Icon(Icons.send),
                onPressed: () => _handleSubmitted(_textController.text),
              ),
            )
          ],
        ),
      ),
    );
  }

  //
  @override
  bool get wantKeepAlive => true;

  //
  void _handleSubmitted(String text) {
    _textController.clear();
    //
    if (text.trim().length == 0) {
      return;
    }
    //
    if (isRobot) {
      BlocProvider.of<MessageBloc>(context)
        ..add(MessageAnswerEvent(
            type: widget.type,
            wid: widget.wid,
            aid: widget.aid,
            content: text));
    } else {
      _bdMqtt.sendTextMessage(text, _currentThread);
    }
  }

  //
  _listener() {
    //
    eventBus.on<ReceiveMessageReceiptEventBus>().listen((event) {
      print('更新状态:' + event.status);
      // TODO: 更新界面
    });
    //
    eventBus.on<ReceiveMessageEventBus>().listen((event) {
      // print('receive message:' + event.message.content);
      // 是否当前会话
      if (event.message.thread.topic != _currentThread.topic) {
        return;
      }
      // 非自己发送的，发送已读回执
      if (event.message.isSend == 0) {
        _bdMqtt.sendReceiptReadMessage(event.message.mid, event.message.thread);
      }
      // 界面显示
      MessageWidget messageWidget = new MessageWidget(
          message: event.message,
          animationController: new AnimationController(
              vsync: this, duration: Duration(milliseconds: 500)));
      setState(() {
        _messages.insert(0, messageWidget);
      });
    });
  }

  Future<void> _getImage() async {
    try {
      PickedFile pickedFile = await picker.getImage(
          source: ImageSource.gallery, maxWidth: 800, imageQuality: 95);
      print('path: ' + pickedFile.path);
      BlocProvider.of<MessageBloc>(context)
        ..add(UploadImageEvent(filePath: pickedFile.path));
    } catch (e) {
      Fluttertoast.showToast(msg: "未选取图片");
    }
  }

  Future<Null> _getMessages() async {
    //
    List<Message> messageList =
        await messageProvider.getTopicMessages(_currentThread.topic);
    messageList.forEach((message) {
      print('mid:' + message.mid);
      MessageWidget messageWidget = new MessageWidget(
          message: message,
          animationController: new AnimationController(
              vsync: this, duration: Duration(milliseconds: 500)));
      setState(() {
        _messages.insert(0, messageWidget);
      });
    });
  }

  Future<Null> _appendMessage(Message message) async {
    print('append:' + message.mid);
    //
    messageProvider.insert(message);
    MessageWidget messageWidget = new MessageWidget(
        message: message,
        animationController: new AnimationController(
            vsync: this, duration: Duration(milliseconds: 500)));
    setState(() {
      _messages.insert(0, messageWidget);
    });
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    print("didChangeAppLifecycleState:" + state.toString());
    switch (state) {
      case AppLifecycleState.inactive: // 处于这种状态的应用程序应该假设它们可能在任何时候暂停。
        break;
      case AppLifecycleState.resumed: // 应用程序可见，前台
        break;
      case AppLifecycleState.paused: // 应用程序不可见，后台
        break;
      case AppLifecycleState.detached: // 申请将暂时暂停
        break;
    }
  }

  @override
  void dispose() {
    WidgetsBinding.instance.removeObserver(this);
    super.dispose();
  }
}
