// import 'package:cached_network_image/cached_network_image.dart';
import 'package:bytedesk_kefu/model/message.dart';
import 'package:bytedesk_kefu/util/constants.dart';
import 'package:flutter/material.dart';

class MessageWidget extends StatelessWidget {
  //
  final Message message;
  final themeColor = Color(0xfff5a623);
  final primaryColor = Color(0xff203152);
  final greyColor = Color(0xffaeaeae);
  final greyColor2 = Color(0xffE8E8E8);
  final AnimationController animationController;

  MessageWidget({this.message, this.animationController});

  @override
  Widget build(BuildContext context) {
    // 头像组件
    return message.isSend == 1
        ? _buildSendWidget()
        : _buildReceiveWidget();
  }

  Widget _buildSendWidget() {
    return new Container(
        margin: EdgeInsets.only(top: 8.0, left: 8.0),
        padding: EdgeInsets.all(8.0),
        child: Column(
          children: <Widget>[
            // 时间戳
            Row(
              mainAxisAlignment: MainAxisAlignment.center,
              children: <Widget>[
                Container(
                  child: Text(
                    message.timestamp ?? '',
                    textAlign: TextAlign.center,
                    style: TextStyle(fontSize: 10.0), // color: Colors.black,
                  ),
                )
              ],
            ),
            // 消息内容
            Row(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                Expanded(
                    child: Row(
                  children: <Widget>[
                    Expanded(flex: 1, child: Container()),
                    // Text(message.status ?? ''), // TODO：发送状态
                    Column(
                      // Column被Expanded包裹起来，使其内部文本可自动换行
                      crossAxisAlignment: CrossAxisAlignment.end,
                      children: <Widget>[
                        Container(
                          padding: EdgeInsets.symmetric(
                              vertical: 8.0, horizontal: 5.0),
                          decoration: BoxDecoration(
                            //image: DecorationImage(image: AssetImage('static/images/chat_bg.png'), fit: BoxFit.fill),
                            borderRadius: BorderRadius.all(
                              Radius.circular(5.0),
                            ),
                            color: Color(0xFF9EEA6A),
                          ),
                          child: message.type == BDConstants.MESSAGE_TYPE_IMAGE
                              ? Image.network(
                                  message.imageUrl,
                                  width: 100,
                                )
                              : Text(
                                  message.content ?? '',
                                  textAlign: TextAlign.right,
                                  style: TextStyle(
                                      color: Colors.black, fontSize: 16.0),
                                ),
                        )
                      ],
                    )
                  ],
                )),
                Image.network(message.avatar, width: 35, height: 35,)
              ],
            ),
          ],
        ));
  }

  Widget _buildReceiveWidget() {
    return new Container(
        margin: EdgeInsets.only(top: 8.0, right: 8.0),
        padding: EdgeInsets.all(8.0),
        child: Column(children: <Widget>[
          Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: <Widget>[
              Container(
                padding: EdgeInsets.symmetric(vertical: 8.0, horizontal: 5.0),
                child: Text(
                  message.timestamp ?? '',
                  // CommonUtils.getTimeDuration(message.timestamp),
                  textAlign: TextAlign.center,
                  style: TextStyle(fontSize: 10.0), // color: Colors.black,
                ),
              )
            ],
          ),
          Row(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              Image.network(message.avatar, width: 35, height: 35,),
              Expanded(
                  child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: <Widget>[
                  Container(
                    padding:
                        EdgeInsets.symmetric(vertical: 8.0, horizontal: 5.0),
                    decoration: BoxDecoration(
                        borderRadius: BorderRadius.all(
                          Radius.circular(5.0),
                        ),
                        color: Color.fromARGB(243, 243, 243, 1)),
                    child: _buildReceivedContent(message),
                  )
                ],
              )),
            ],
          ),
        ]));
  }

  // 接收消息体
  Widget _buildReceivedContent(Message message) {
    //
    if (message.type == BDConstants.MESSAGE_TYPE_TEXT) {
      return Text(
        message.content ?? '',
        textAlign: TextAlign.left,
        style: TextStyle(color: Colors.black, fontSize: 16.0),
      );
    } else if (message.type == BDConstants.MESSAGE_TYPE_IMAGE) {
      return Image.network(
        message.imageUrl,
        width: 100,
      );
    } else if (message.type == BDConstants.MESSAGE_TYPE_ROBOT) {
      return Column(
        children: <Widget>[
          Text(
            message.content ?? '',
            textAlign: TextAlign.left,
            style: TextStyle(color: Colors.black, fontSize: 16.0),
          ),
          ListView.builder(
            // 如果滚动视图在滚动方向无界约束，那么shrinkWrap必须为true
            shrinkWrap: true,
            // 禁用ListView滑动，使用外层的ScrollView滑动
            physics: const NeverScrollableScrollPhysics(),
            itemCount: message.answers == null ? 0 : message.answers.length,
            itemBuilder: (_, index) {
              //
              var answer = message.answers[index];
              return new InkWell(
                  child: new Text(answer.question),
                  onTap: () => {print('object:' + answer.question)});
            },
          )
        ],
      );
    } else {
      return Text(
        message.content ?? '',
        textAlign: TextAlign.left,
        style: TextStyle(color: Colors.black, fontSize: 16.0),
      );
    }
  }
}
